
/* Chrysalide - Outil d'analyse de fichiers binaires
 * virtual.h - prototypes pour l'encadrement des instructions par blocs virtuels
 *
 * Copyright (C) 2012-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_BLOCKS_VIRTUAL_H
#define _ANALYSIS_BLOCKS_VIRTUAL_H


#include <glib.h>
#include <glib-object.h>


#include "../block.h"
#include "../../arch/instruction.h"



#define G_TYPE_VIRTUAL_BLOCK               g_virtual_block_get_type()
#define G_VIRTUAL_BLOCK(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_virtual_block_get_type(), GVirtualBlock))
#define G_IS_VIRTUAL_BLOCK(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_virtual_block_get_type()))
#define G_VIRTUAL_BLOCK_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_VIRTUAL_BLOCK, GVirtualBlockClass))
#define G_IS_VIRTUAL_BLOCK_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_VIRTUAL_BLOCK))
#define G_VIRTUAL_BLOCK_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_VIRTUAL_BLOCK, GVirtualBlockClass))


/* Description d'un bloc virtuel d'instructions (instance) */
typedef struct _GVirtualBlock GVirtualBlock;

/* Description d'un bloc virtuel d'instructions (classe) */
typedef struct _GVirtualBlockClass GVirtualBlockClass;


/* Indique le type défini pour un bloc virtuel d'instructions. */
GType g_virtual_block_get_type(void);

/* Crée un bloc virtuel d'instructions. */
GInstrBlock *g_virtual_block_new(void);

/* Ajoute un bloc au groupe courant. */
void g_virtual_block_add_child(GVirtualBlock *, GInstrBlock *);

/* Compte le nombre de blocs contenus dans le groupe courant. */
size_t g_virtual_block_count_children(const GVirtualBlock *);

/* Renvoie un des blocs contenus dans le groupe courant. */
GInstrBlock *g_virtual_block_get_child(const GVirtualBlock *, size_t);



#endif  /* _ANALYSIS_BLOCKS_VIRTUAL_H */
